	local L = LibStub("AceLocale-3.0"):GetLocale("sqwNameplates")
	local LSM = LibStub("LibSharedMedia-3.0")
	local t,t2

	--[[ Print ]]
	local function p(text)
		ChatFrame3:AddMessage(tostring(text))
	end
	local u = update
	local UnitAura = UnitAura
	local UnitExists = UnitExists
	local strlower = strlower
	local unpack = unpack
	local tonumber = tonumber
	local tostring = tostring
	local ipairs = ipairs
	local next = next
	local pairs	= pairs
	local sort = sort
	local strfind = strfind
	local tremove = tremove
	local tinsert = tinsert
	local UnitGUID = UnitGUID
	local UnitBuff = UnitBuff
	local UnitDebuff = UnitDebuff
	local UnitIsPlayer = UnitIsPlayer
	local UnitClassification = UnitClassification
	local format = format
	local floor = floor

	local db, level, guidCasts
	local resources = sqwNameplates.resources
	local frameNumberCounter = 0
	local INVISIBLE_TEXTURE = "Interface\\Addons\\sqwNameplates\\media\\invisible2x2"
	local spacing
	local backdrop


	function round(num, idp) --number, decimal places
		local mult = 10^(idp or 0)
		return math.floor(num * mult + 0.5) / mult
	end

	local function getSocialColor(social)
		if	   social=="raid"		then   return 255/255, 127/255, 0/255
		elseif social=="party"		then   return 166/255, 166/255, 255/255
		elseif social=="guild"		then   return 64/255, 255/255, 64/255
		elseif social=="bnet"		then   return 115/255, 199/255, 255/255
		elseif social=="friend"		then   return 254/255, 222/255, 41/255
	else					       		   return 255/255, 255/255, 255/255
			
		end
	end

	local function formatTime(time)
		if time>60 then time = floor(time/60).."m"
		elseif time>=3 then time = floor(time)
		else time = round(time, 1)
		end
		return time
	end

	local function createImprovedStatusbar(self, existingBar) -- Creates a tex that acts like a statusbar. This gets rid of the ugly tiling.
		local function UpdateBar(self)
			local fraction = .01
			local range = self.MaxVal-self.MinVal 
			local value = self.Value-self.MinVal
			local barsize = self.Dim or 100
			
			if range>0 and value>0 and range>=value then
				fraction = value/range
			else
				fraction = .0001
			end
			
			if self.Orientation=="VERTICAL" then 
				self.Bar:SetHeight(barsize*fraction)
				self.Bar:SetTexCoord(0, 1, 1-fraction, 1)
			else 
				self.Bar:SetWidth(barsize*fraction) 
				self.Bar:SetTexCoord(0, fraction, 0, 1)
			end
		end
		
		local function UpdateSize(self) 
			if self.Orientation=="VERTICAL" then
				self.Dim = self:GetHeight()
			else
				self.Dim = self:GetWidth()
			end
			
			UpdateBar(self) 
		end
		
		local function SetValue(self, value)
			if value>=self.MinVal and value<=self.MaxVal then self.Value = value end
			
			UpdateBar(self) 
		end
		
		local function SetMinMaxValues(self, minval, maxval)
			if maxval>minval then
				self.MinVal = minval
				self.MaxVal = maxval
			else 
				self.MinVal = 0
				self.MaxVal = 100
			end
			
			if self.Value>self.MaxVal then self.Value = self.MaxVal
			elseif self.Value<self.MinVal then self.Value = self.MinVal end
			
			UpdateBar(self) 
		end
		
		local function SetOrientation(self, orientation) 
			if orientation=="VERTICAL" then
				self.Orientation = orientation
				self.Bar:ClearAllPoints()
				self.Bar:SetPoint("BOTTOMLEFT")
				self.Bar:SetPoint("BOTTOMRIGHT")
			else
				self.Orientation = "HORIZONTAL"
				self.Bar:ClearAllPoints()
				self.Bar:SetPoint("TOPLEFT")
				self.Bar:SetPoint("BOTTOMLEFT")
			end
			
			UpdateSize(self)
		end
		
		local function SetStatusBarColor(self, r, g, b, a)
			self.Bar:SetVertexColor(r, g, b, a)
		end
		
		local function SetStatusBarTexture(self, tex)
			self.Bar:SetTexture(tex)
		end
		
		local function SetDrawLayer(self, layer)
			self.Bar:SetDrawLayer(layer)
		end
		
		local statusbar
		if not existingBar then
			statusbar = CreateFrame("Frame", nil, self)
		else
			statusbar = existingBar
		end
		statusbar:SetWidth(db.width)
		statusbar:SetHeight(db.height)
		
		if not statusbar.Bar then
			t = statusbar:CreateTexture(nil, "ARTWORK")
		else
			t = statusbar.Bar
		end
		statusbar.Bar = t
		
		if not statusbar.border then
			statusbar.border = CreateFrame("Frame", nil, statusbar)
		end
		statusbar.border:SetPoint("TOPLEFT", statusbar, "TOPLEFT", -db.borderSize, db.borderSize)
		statusbar.border:SetPoint("BOTTOMRIGHT", statusbar, "BOTTOMRIGHT", db.borderSize, -db.borderSize)
		
		statusbar.Value = 100
		statusbar.MinVal = 0
		statusbar.MaxVal = 100
		statusbar.Orientation = "HORIZONTAL"
		statusbar.UpdateSize = UpdateSize
		statusbar.SetValue = SetValue
		statusbar.SetMinMaxValues = SetMinMaxValues
		statusbar.SetOrientation = SetOrientation
		statusbar.SetStatusBarColor = SetStatusBarColor
		statusbar.SetStatusBarTexture = SetStatusBarTexture
		statusbar.SetDrawLayer = SetDrawLayer
		
		statusbar:SetFrameLevel(level)
		statusbar:SetOrientation("HORIZONTAL")
		statusbar:SetStatusBarTexture(LSM:Fetch("statusbar", db.tex))
		
		statusbar:HookScript("OnSizeChanged", UpdateSize)
		UpdateSize(statusbar)
		
		return statusbar
	end

	--[[ Refresh nameplate layouts ]]
	function sqwNameplates:refreshNameplateLayouts()
		for i=1,#sqwNameplates.frames do
			sqwNameplates.createNewNameplate(sqwNameplates.frames[i], "refresh")
			sqwNameplates.onHideNameplate(sqwNameplates.frames[i])
		end
		
		for i=1,#sqwNameplates.frames do
			if sqwNameplates.frames[i]:IsShown() then
				sqwNameplates.onShowNameplate(sqwNameplates.frames[i])
			end
		end
		
		sqwNameplates:setVisibility()
	end

	function sqwNameplates:OnProfileChange()
		sqwNameplates:initializeUpdater()
		sqwNameplates:refreshNameplateLayouts()
		
		-- Choose preset
		if not db.presetChosen or db.presetChosen<sqwNameplates.version then
			sqwNameplates.LoadPreset()
		end
	end

	--[[ Create nameplate elements ]]
	local create = {}
	function create:test()
	end

	-- Name
	function create:name()
		local fontSize = db.fontsizeName
		
		if not self.name then
			t = self:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		else
			t = self.name
		end
		
		t:SetPoint("TOP", self, "TOP", 0, 0)
		t:SetFont(LSM:Fetch("font", db.font), fontSize, db.outline)
		t:SetJustifyH("CENTER")
		t:SetJustifyV("CENTER")
		t:SetShadowColor(0,0,0,1)
		t:SetShadowOffset(1,-1)
		self.name = t
	end

	-- ToT
	function create:tot()
		local fontSize = db.fontsize-2
		
		if not self.tot then
			t = self:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		else
			t = self.tot
		end
		t:SetWidth(db.width*4)
		t:SetHeight(fontSize)
		t:ClearAllPoints()
		t:SetPoint("BOTTOM", self.name, "TOP", 0, 1)
		t:SetFont(LSM:Fetch("font", db.font), fontSize)
		t:SetJustifyH("CENTER")
		t:SetJustifyV("CENTER")
		t:SetShadowColor(0,0,0,1)
		t:SetShadowOffset(1,-1)
		--t:Hide()
		self.tot = t
	end

	--Guild
	function create:guild()
		local fontSize = db.fontsizeGuild
	
		if not self.guild then
			t = self:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		else
			t = self.guild
		end
		t:SetWidth(60*4)
		t:SetHeight(fontSize)
		t:ClearAllPoints()
		t:SetPoint("TOP", self.name, "BOTTOM", 0, 0) 
		t:SetFont(LSM:Fetch("font", db.fonti), fontSize, db.outline)
		t:SetJustifyH("CENTER")
		t:SetJustifyV("CENTER")
		t:SetShadowColor(0,0,0,1)
		t:SetShadowOffset(1,-1)
		self.guild = t
	end

	--HP
	function create:hp()
	db.bgColor.a = 0.2
		if not self.hpNumber then
			t = self:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		else
			t = self.hpNumber
		end
	   
		t:SetPoint("TOP", self.name, "BOTTOM", 0, -4)
		t:SetFont(LSM:Fetch("font", db.fontb), db.fontsizeHealth)
		t:SetJustifyH("CENTER")
		t:SetJustifyV("CENTER")
		t:SetShadowColor(0,0,0,1)
		t:SetShadowOffset(1,-1)
		self.hpNumber = t
		
		if not self.hpNumberBg then
			t = self:CreateTexture(nil, "BORDER")
		else
			t = self.hpNumberBg
			
		end
		t:SetAllPoints(self.hpNumber)
		t:SetTexture("Interface\\Buttons\\WHITE8X8")
		t:SetVertexColor(db.bgColor.r,db.bgColor.g,db.bgColor.b,db.bgColor.a)
		t.visible = false

		t:Hide()
		self.hpNumberBg = t
		
		self.hp = createImprovedStatusbar(self, self.hp)
		self.hp:SetWidth(db.width)
		self.hp:SetHeight(db.height)
		self.hp:SetPoint("CENTER", self.hpNumber, "CENTER", 0, 0)
		self.hp.border:SetBackdrop(backdrop)
		self.hp.border:SetBackdropBorderColor(db.borderColor.r,db.borderColor.g,db.borderColor.b,db.borderColor.a)
		self.hp:SetStatusBarColor(1, 1, 1, 1)
		self.hp:UpdateSize(self.hp)
		self.hp.visible = false
		self.hp:Hide()
		
		if not self.hpBg then
			t = self.hp:CreateTexture(nil, "BORDER")
		else
		   t = self.hpBg
			
		end
		t:SetAllPoints(self.hp)
		t:SetTexture("Interface\\Buttons\\WHITE8X8")
		t:SetVertexColor(db.bgColor.r,db.bgColor.g,db.bgColor.b,db.bgColor.a)
		self.hpBg = t
	end

	--LEVEL
	function create:level()
		
		local fontSize = db.fontsizeName
		
		if not self.level then
			t = self:CreateFontString(nil, "BORDER", "GameFontNormal")
		else
			t = self.level
		end
		if db.showHpBar then
			t:SetPoint("LEFT", self.hp, "RIGHT", spacing, 0) --orig position right of health value
			--t:SetPoint("LEFT", self.name, "RIGHT", spacing, 0) -- position right of name
		else
			--t:SetPoint("LEFT", self.hpNumber, "RIGHT", spacing, 0) --orig position right of health number
			t:SetPoint("LEFT", self.name, "RIGHT", spacing, 0) -- position right of name
		end
		t:SetFont(LSM:Fetch("font", db.fontb), fontSize, db.outline)
		t:SetJustifyH("LEFT")
		t:SetJustifyV("CENTER")
		t:SetShadowColor(0,0,0,1)
		t:SetShadowOffset(1,-1)
		self.level = t
	end

	--BUFFS
	local function buffOnUpdate(self, elapsed)
		self.lastUpdate = self.lastUpdate+elapsed
		
		if self.lastUpdate>0.1 then
			local t = GetTime()
			
			if self.count>1 and self.counter.saved~=self.count then
				self.counter:SetText(self.count)
				self.counter.saved = self.count
			elseif not self.count or self.count<=1 then
				self.counter:SetText(nil)
				self.counter.saved = -1
			end
			
			if self.expirationTime>0 then
				if t>self.expirationTime then
					self:Hide()
					self.visible = false
					
					self.frame:u("onlyAura")
				else
					local time = formatTime(self.expirationTime-t)
					
					if time~=self.time.saved then
						self.time:SetText(time)
						self.time.saved = time
					end
				end
			else
				self.time:SetText(nil)
			end
			
			self.lastUpdate = 0
		end
	end
	function create:buffs(frame)
		local np = self
		self.buffs = self.buffs or {}
		local list = self.buffs
		
		for i=1,db.buffsNumber do
			local buff
			if not list[i] then
				buff = CreateFrame("Frame", nil, self)
				tinsert(list, buff)
			else
				buff = list[i]
			end
			buff:SetWidth(12)
			buff:SetHeight(12)
			buff:ClearAllPoints()
			if i==1 then
				if db.showHpBar then
					buff:SetPoint("RIGHT", self.hp, "LEFT", 3, 0)
				else
					buff:SetPoint("RIGHT", self.hpNumber, "LEFT", 3, 0)
				end
			else
				buff:SetPoint("RIGHT", list[i-1], "LEFT", -spacing*2, 0)
			end
			buff:SetScale(db.scaleBuffs)
			buff.frame = frame
			
			local icon
			if not buff.icon then
				icon = buff:CreateTexture(nil, "ARTWORK")
			else
				icon = buff.icon
			end
			icon:ClearAllPoints()
			icon:SetAllPoints(buff)
			icon:SetTexCoord(0.1,0.9,0.1,0.9)
			buff.icon = icon
			
			local time
			if not buff.time then
				time = buff:CreateFontString(nil, "ARTWORK", "GameFontNormal")
			else
				time = buff.time
			end
			time:ClearAllPoints()
			time:SetPoint("CENTER", buff, "BOTTOM", 0, 0)
			time:SetFont(LSM:Fetch("font", db.font), db.fontsizeBuffs, "THINOUTLINE")
			time:SetJustifyH("CENTER")
			time:SetJustifyV("CENTER")
			time:SetShadowColor(0,0,0,1)
			time:SetShadowOffset(1,-1)
			time:SetTextColor(db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a)
			buff.time = time
			
			local counter
			if not buff.counter then
				counter = buff:CreateFontString(nil, "ARTWORK", "GameFontNormal")
			else
				counter = buff.counter
			end
			counter:ClearAllPoints()
			counter:SetPoint("CENTER", buff, "TOP", 0, spacing)
			counter:SetFont(LSM:Fetch("font", db.fontb), db.fontsizeBuffs, "THINOUTLINE")
			counter:SetJustifyH("CENTER")
			counter:SetJustifyV("CENTER")
			counter:SetShadowColor(0,0,0,1)
			counter:SetShadowOffset(1,-1)
			counter:SetTextColor(db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a)
			counter.saved = -1
			buff.counter = counter
			
			buff.index = nil
			
			buff.lastUpdate = 0
			buff:HookScript("OnUpdate", buffOnUpdate)
			
			buff.expirationTime = 0
			buff:Hide()
			buff.visible = false
		end
	end

	--DEBUFFS
	function create:debuffs(frame)
		local np = self
		self.debuffs = self.debuffs or {}
		local list = self.debuffs
		
		for i=1,db.buffsNumber do
			local buff
			if not list[i] then
				buff = CreateFrame("Frame", nil, self)
				tinsert(list, buff)
			else
				buff = list[i]
			end
			buff:SetWidth(12)
			buff:SetHeight(12)
			buff:ClearAllPoints()
			if i==1 then
				if db.showHpBar then
					buff:SetPoint("LEFT", self.hp, "RIGHT", 10, 0)
				else
					buff:SetPoint("LEFT", self.hpNumber, "RIGHT", 10, 0)
				end
			else
				buff:SetPoint("LEFT", list[i-1], "RIGHT", spacing*2, 0)
			end
			buff:SetScale(db.scaleBuffs)
			buff.frame = frame
			
			local icon
			if not buff.icon then
				icon = buff:CreateTexture(nil, "ARTWORK")
			else
				icon = buff.icon
			end
			icon:ClearAllPoints()
			icon:SetAllPoints(buff)
			icon:SetTexCoord(0.1,0.9,0.1,0.9)
			buff.icon = icon
			
			local time
			if not buff.time then
				time = buff:CreateFontString(nil, "ARTWORK", "GameFontNormal")
			else
				time = buff.time
			end
			time:ClearAllPoints()
			time:SetPoint("CENTER", buff, "BOTTOM", 0, 0)
			time:SetFont(LSM:Fetch("font", db.font), db.fontsizeBuffs, "THINOUTLINE")
			time:SetJustifyH("CENTER")
			time:SetJustifyV("CENTER")
			time:SetShadowColor(0,0,0,1)
			time:SetShadowOffset(1,-1)
			time:SetTextColor(db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a)
			buff.time = time
			
			local counter
			if not buff.counter then
				counter = buff:CreateFontString(nil, "ARTWORK", "GameFontNormal")
			else
				counter = buff.counter
			end
			counter:ClearAllPoints()
			counter:SetPoint("CENTER", buff, "TOP", 0, spacing)
			counter:SetFont(LSM:Fetch("font", db.fontb), db.fontsizeBuffs, "THINOUTLINE")
			counter:SetJustifyH("CENTER")
			counter:SetJustifyV("CENTER")
			counter:SetShadowColor(0,0,0,1)
			counter:SetShadowOffset(1,-1)
			counter:SetTextColor(db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a)
			counter.saved = -1
			buff.counter = counter
			
			buff.index = nil
			
			buff.lastUpdate = 0
			buff:HookScript("OnUpdate", buffOnUpdate)
			
			buff.expirationTime = 0
			buff:Hide()
			buff.visible = false
		end
	end


	--CASTING
	function create:cast() --LIST ERR
		--[[ local function onUpdate(self, elapsed)
		self.lastUpdate = self.lastUpdate+elapsed
		if self.lastUpdate>0.025 then]]

		self.lastUpdate=0
		
		local function onUpdate(self, elapsed)
			self.lastUpdate = self.lastUpdate+elapsed;
			while (self.lastUpdate > 0.025) do

				self.list.value = self.list.value+self.lastUpdate
				self:SetValue(self.list.value)
				self.time:SetText(resources.round(self.list.value, 1))
				self.lastUpdate = 0 --orig
			end
			
			if self.list.value>10 or self.list.value>self.list.max then
				if self.visible then
					self:Hide()
					self.visible = false
				end
				
				self.list.isCasting = false
				self.lastUpdate = 0
			end
		end
		
		local function onShow(self)
			
		end
		
		local function onHide(self)
			
		end
		
		local fontSize = db.fontsizeCast
		
		self.cast = createImprovedStatusbar(self, self.cast)
		self.cast:SetWidth(db.widthCast)
		self.cast:SetHeight(db.heightCast)
		self.cast:SetPoint("TOP", self.hpNumber, "BOTTOM", 0, -spacing*2)
		self.cast.border:SetBackdrop(backdrop)
		self.cast.border:SetBackdropBorderColor(db.borderColor.r,db.borderColor.g,db.borderColor.b,db.borderColor.a)
		self.cast:SetStatusBarColor(db.castColor.r,db.castColor.g,db.castColor.b,db.castColor.a)
		self.cast:UpdateSize()
		self.cast:HookScript("OnUpdate", onUpdate)
		self.cast.lastUpdate = 0
		
		if not self.cast.bg then
			t = self.cast:CreateTexture(nil, "BACKGROUND")
		else
			t = self.cast.bg
		end
		t:SetAllPoints(self.cast)
		t:SetTexture(LSM:Fetch("statusbar", db.tex))
		t:SetVertexColor(db.bgColor.r, db.bgColor.g, db.bgColor.b, 0.5)
		self.cast.bg = t
		
		if not self.cast.time then
			t = self.cast:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		else
			t = self.cast.time
		end
		--t:SetWidth(60)
		--t:SetHeight(db.height)
		t:SetPoint("LEFT", self.cast, "RIGHT", spacing, 0)
		t:SetFont(LSM:Fetch("font", db.fontb), fontSize-1, db.outline)
		t:SetJustifyH("LEFT")
		t:SetJustifyV("CENTER")
		t:SetShadowColor(0,0,0,1)
		t:SetShadowOffset(1,-1)
		t:SetTextColor(db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a)
		self.cast.time = t
		
		if not self.cast.icon then
			t = CreateFrame("Frame", nil, self.cast)
		else
			t = self.cast.icon
		end
		t:SetWidth(16)
		t:SetHeight(16)
		t:SetPoint("TOPRIGHT", self.cast, "TOPLEFT", -spacing*2, 0)
	   
		self.cast.icon = t
		
		if not self.cast.icon.tex2 then
			t = self.cast.icon:CreateTexture(nil, "BORDER")
		else
			t = self.cast.icon.tex2
		end
		t:SetAllPoints(self.cast.icon)
		t:SetTexCoord(0.1,0.9,0.1,0.9)
		t:SetTexture("Interface\\Icons\\Inv_misc_questionmark")
		self.cast.icon.tex2 = t
		
		if not self.cast.icon.tex then
			t = self.regions.spellIcon
		else
			t = self.cast.icon.tex
		end
		--t:SetParent(self.cast.icon)
		--t:SetDrawLayer("ARTWORK")
		--t:SetAllPoints(self.cast.icon)
		--t:SetTexCoord(0.1,0.9,0.1,0.9)
		--self.cast.icon.tex = t
		
		if not self.cast.text then
			t = self.cast:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		else
			t = self.cast.text
		end
		--t:SetWidth(120)
		--t:SetHeight(fontSize)
		t:SetPoint("TOPLEFT", self.cast, "BOTTOMLEFT", 0, -spacing)
		t:SetFont(LSM:Fetch("font", db.font), fontSize-2, db.outline)
		t:SetJustifyH("LEFT")
		t:SetJustifyV("CENTER")
		t:SetShadowColor(0,0,0,1)
		t:SetShadowOffset(1,-1)
		t:SetTextColor(db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a)
		self.cast.text = t
	end

	--RAID ICON
	function create:raidIcon() --above name
		if not self.raidIcon then
			t = self:CreateTexture(nil, "BORDER")
		else
			t = self.raidIcon
		end
		t:SetWidth(25)
		t:SetHeight(25)
		t:SetPoint("BOTTOM", self.name, "TOP", 0, 5)
		t:SetTexture(self.regions.raidIcon:GetTexture())
		t:Hide()
		self.raidIcon = t
	end

	--CLASS ICON
	function create:classIcon()
		if not self.classIcon then
			t = self:CreateTexture(nil, "BORDER")
		else
			t = self.classIcon
		end
		t:SetWidth(12)
		t:SetHeight(12)
		t:SetPoint("RIGHT", self.name, "LEFT", -spacing, 0)
		t:SetTexture("Interface\\WorldStateFrame\\Icons-Classes")
		t:Hide()
		self.classIcon = t
	end

	--ROLE ICON
	function create:roleIcon()
		if not self.roleIcon then
			t = self:CreateTexture(nil, "BORDER")
		else
			t = self.roleIcon
		end
		t:SetWidth(12)
		t:SetHeight(12)
		t:SetPoint("RIGHT", self.hp, "LEFT", -spacing, 0)
		t:SetTexture("Interface\\LFGFrame\\LFGRole")
		t:Hide()
		self.roleIcon = t
	end

	--TOTEM ICON
	function create:totemIcon()
		if not self.totemIcon then
			t = self:CreateTexture(nil, "BORDER")
		else
			t = self.totemIcon
		end
		t:SetWidth(25)
		t:SetHeight(25)
		t:SetPoint("TOP", self, "TOP", 0, 0)
		t:SetTexture("Interface\\Icons\\Mail_gmicon")
		t:SetTexCoord(0.1,0.9,0.1,0.9)
		t:Hide()
		self.totemIcon = t
	end

	--UNKNOWN ICON
	function create:unknownIcon()
		if not self.unknownIcon then
			t = self:CreateTexture(nil, "BORDER")
		else
			t = self.unknownIcon
		end
		t:SetWidth(20)
		t:SetHeight(20)
		t:SetPoint("CENTER", self.name, "TOP", 0, spacing*2)
		t:SetTexture("Interface\\Addons\\sqwNameplates\\media\\unknown")
		t:SetVertexColor(0.25,0.99,0.99,1)
		t:Hide()
		self.unknownIcon = t
	end

	--TARGET NHL
	function create:targetNHL()
		if not self.targetNHL then
			t = self:CreateTexture(nil, "HIGHLIGHT")
		else
			t = self.targetNHL
		end
		
		t:SetTexture("Interface\\Addons\\sqwNameplates\\media\\nhl")
		t:SetPoint("TOPLEFT", self.name, "TOPLEFT", -7, 5)
		t:SetPoint("BOTTOMRIGHT", self.level, "BOTTOMRIGHT", 5, -3)
		t:SetPoint("BOTTOMRIGHT", self.level, "BOTTOMRIGHT", 18, 0)
		t:SetVertexColor(0.5, 0.5, 0, 0.75)
		t:Hide()
		self.targetNHL = t
	end

	--TARGET NHL2
	function create:targetNHL2()
		if not self.targetNHL2 then
			t = self:CreateTexture(nil, "HIGHLIGHT")
		else
			t = self.targetNHL2
		end
			
		t:SetTexture("Interface\\Addons\\sqwNameplates\\media\\unknown")
		t:SetPoint("CENTER", self.name, "TOP", 0, 10)
		t:SetVertexColor(1, .6, .3, 1)
		t:Hide()
		self.targetNHL2 = t
	end

	--TARGET ICON
	function create:targetIcon()
		if not self.targetIcon then
			t = self:CreateTexture(nil, "OVERLAY")
		else
			t = self.targetIcon
		end
		
		t:SetWidth(40)
		t:SetHeight(40)
		t:SetPoint("BOTTOM", self.name, "TOP", 0, 5)
		t:SetTexture("Interface\\Addons\\sqwNameplates\\media\\ti")
		t:SetVertexColor(1, 1, 1, 1)
		t:Hide()
		self.targetIcon = t
	end

	--SOCIAL ICON
	function create:socialIcon()
		if not self.socialIcon then
			t = self:CreateTexture(nil, "BORDER")
		else
			t = self.socialIcon
		end
		t:SetWidth(25)
		t:SetHeight(25)
		t:SetPoint("BOTTOM", self.name, "TOP", 0, -spacing*2)
		t:SetTexture("Interface\\Addons\\sqwNameplates\\media\\social")
		t:SetVertexColor(1, 1, 1, 1)
		t:Hide()
		self.socialIcon = t
	end


	--THREAT
	function create:threat()
		
	end

	--TARGET HIGHLIGHT
	function create:targetHighlight()
		if not self.targetHighlight then
			t = self:CreateTexture(nil, "OVERLAY")
		else
			t = self.targetHighlight
		end
		 t:SetTexture("Interface\\Addons\\sqwNameplates\\media\\highlightTarget") --orig
		--t:SetTexture("Interface\\Addons\\sqwNameplates\\media\\SoftOutline")
		t:SetPoint("TOPLEFT", self.hpNumber, "TOPLEFT", -2, 2)
		t:SetPoint("BOTTOMRIGHT", self.hpNumber, "BOTTOMRIGHT", 2, -2)
		--t:SetVertexColor(0.5, 0.5, 0, 0.75) --yellow border around health value
		t:SetVertexColor(1,1,1,1) --white border around health value
		t:Hide()
		self.targetHighlight = t
	end

	--COMBO
	function create:combo()
		if true then return end
		
		local size = 10
		
		if not self.combo then
			self.combo = {}
		end
		local combo = self.combo
		
		t = self:CreateTexture(nil, "OVERLAY")
		t:SetTexture("Interface\\Addons\\sqwNameplates\\media\\combo")
		t:SetWidth(size)
		t:SetHeight(size)
		t:SetPoint("BOTTOM", self.name, "BOTTOM", 0, 0)
		t:SetVertexColor(1,0.9,0,1)
		--t:Hide()
		combo[3] = t
		
		t = self:CreateTexture(nil, "OVERLAY")
		t:SetTexture("Interface\\Addons\\sqwNameplates\\media\\combo")
		t:SetWidth(size)
		t:SetHeight(size)
		t:SetPoint("RIGHT", combo[3], "LEFT", 0, 0)
		t:SetVertexColor(1,0.9,0,1)
		--t:Hide()
		combo[2] = t
		
		t = self:CreateTexture(nil, "OVERLAY")
		t:SetTexture("Interface\\Addons\\sqwNameplates\\media\\combo")
		t:SetWidth(size)
		t:SetHeight(size)
		t:SetPoint("RIGHT", combo[2], "LEFT", 0, 0)
		t:SetVertexColor(1,0.9,0,1)
		--t:Hide()
		combo[1] = t
		
		t = self:CreateTexture(nil, "OVERLAY")
		t:SetTexture("Interface\\Addons\\sqwNameplates\\media\\combo")
		t:SetWidth(size)
		t:SetHeight(size)
		t:SetPoint("LEFT", combo[3], "RIGHT", 0, 0)
		t:SetVertexColor(1,0.9,0,1)
		--t:Hide()
		combo[4] = t
		
		t = self:CreateTexture(nil, "OVERLAY")
		t:SetTexture("Interface\\Addons\\sqwNameplates\\media\\combo")
		t:SetWidth(size)
		t:SetHeight(size)
		t:SetPoint("LEFT", combo[4], "RIGHT", 0, 0)
		t:SetVertexColor(1,0.9,0,1)
		--t:Hide()
		combo[5] = t
		
	end

	--DEBUG
	function create:debug()
		-- self = self
		
		self.debug = self.debug or {}
		local debug = self.debug
		
		local fontSize = db.fontsizeGuild
		
		if not debug.text then
			t = self:CreateFontString(nil, "OVERLAY")
		else
			t = debug.text
		end
		t:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -spacing)
		t:SetFont(LSM:Fetch("font", db.font), fontSize)
		t:SetJustifyH("LEFT")
		t:SetJustifyV("CENTER")
		t:SetShadowColor(0,0,0,1)
		t:SetShadowOffset(1,-1)
		t:SetTextColor(db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a)
		debug.text = t
		
		if not debug.bg then
			t = self:CreateTexture(nil, "BORDER")
		else
			t = debug.bg
		end
		t:SetAllPoints(self)
		t:SetTexture(LSM:Fetch("statusbar", db.tex))
		t:SetVertexColor(db.fontColor.r,db.fontColor.g,db.fontColor.b,0.33)
		t:Hide()
		debug.bg = t
	end

	--CREATE NAMEPLATE FRAMES
	sqwNameplates.frames = {}

	function sqwNameplates:createNewNameplate(refresh)
		local frame = self
		level = frame:GetFrameLevel()
		
		db = sqwNameplates.db.profile
		spacing = db.spacingFixed
		guidCasts = sqwNameplates.GUIDCasts
		local edgeSize
		if db.border=="Tusqw" or db.border=="LUI" then edgeSize = 3
		else edgeSize = 8 end
		backdrop = { 
			bgFile = nil, 
			edgeFile = LSM:Fetch("border", db.border), tile = true, tileSize = 16, edgeSize = edgeSize, 
			insets = { left = 4, right = 4, top = 4, bottom = 4 },
		} 
		
		--MAIN NAMEPLATE FRAME
		local np
		if not frame.np then
			np = CreateFrame("Frame", nil, frame)
			--np:SetFrameStrata("BACKGROUND")
			np:SetAlpha(1)
			np:SetAllPoints(frame)
			--np:EnableDrawLayer("HIGHLIGHT") -sqw remmed 22-09
			
			
			
			np.d = {}
			np.savedData = {}
			np.originalWidth = floor(221)
			np.originalHeight = floor(55) 
			np.alpha = 1
			if not np.number then
				frameNumberCounter = frameNumberCounter+1
				np.number = frameNumberCounter
			end
			np.uCounter = 0
			frame.u = sqwNameplates.uNameplate
			frame.onShow = sqwNameplates.onShowNameplate
			frame.onHide = sqwNameplates.onHideNameplate
			frame.setDoUpdateMe = sqwNameplates.setDoUpdateMe
			
			
			--ORIGINAL NAMEPLATE
			np.children = {}
			local child = np.children
			np.regions = {}
			
			
			frame.barFrame, frame.nameFrame = frame:GetChildren()
			child.healthBar, _,child.castBar = frame.barFrame:GetChildren()
			np.regions.spelltext = frame.nameFrame:GetRegions()
						
			    np.regions.threatglow,
				np.regions.healthborder,
				np.regions.highlight,
				np.regions.level,
				np.regions.skullicon, 
				np.regions.raidIcon,
				np.regions.eliteicon = frame.barFrame:GetRegions()
			_,
				np.regions.castBorder,
				np.regions.castNostop,
				np.regions.spellIcon, 
				frame.spelltext, 
				frame.spellshadow = child.castBar:GetRegions()
			
			child.healthBar:SetStatusBarTexture(INVISIBLE_TEXTURE)
			child.castBar:SetStatusBarTexture(INVISIBLE_TEXTURE)

			-----
			np.regions.threatglow:SetTexture(INVISIBLE_TEXTURE)
			--np.highlight:SetTexture(INVISIBLE_TEXTURE) 
			np.regions.skullicon:SetTexture(INVISIBLE_TEXTURE)
			frame.spellshadow:SetTexture(INVISIBLE_TEXTURE)
			np.regions.castBorder:SetTexture(INVISIBLE_TEXTURE)
			np.regions.castNostop:SetTexture(INVISIBLE_TEXTURE)
			------------
			
			
			--np.regions.threatglow:SetTexture(false)
			np.regions.healthborder:Hide()
			np.highlight = np.regions.highlight
			np.highlight:SetTexture(false) 
			np.regions.level:SetWidth(000.1)
			np.regions.level:Hide()
			--np.regions.skullicon:SetTexture(false)
			np.regions.raidIcon:SetAlpha(0)		
			np.regions.eliteicon:SetAlpha(0)
			frame.nameFrame:Hide()
			np.regions.castBorder:Hide()
			np.regions.castBorder:SetVertexColor(1,1,1,1)
			np.highlight:SetVertexColor(1,1,1,1)
			np.regions.spelltext:SetWidth(000.1)
			frame.spelltext:Hide()					
			--frame.spellshadow:SetTexture(false)
			--np.regions.castBorder:SetTexture(false)
			--np.regions.castNostop:SetTexture(false)
			np.regions.spellIcon:SetTexCoord( 0, 0, 0, 0 )
			np.regions.spellIcon:SetWidth(.001)
			np.regions.spellIcon:Hide()
		
			

			--HOOKS
			frame:HookScript("OnShow", function()
				frame:onShow()
			end)
			frame:HookScript("OnHide", function()
				frame:onHide()
			end)
			
			child.healthBar:HookScript("OnValueChanged", function()
				frame:setDoUpdateMe("frame", "onlyHp")
			end)
			
		else
			np = frame.np
		end
		frame.np = np
		np:SetScale(db.scale)
		
		create.name(np)
		create.tot(np)
		create.guild(np)
		create.hp(np)
		create.level(np)
		create.threat(np)
		create.targetHighlight(np)
		create.buffs(np, frame)
		create.debuffs(np, frame)
		create.cast(np)
		create.targetNHL(np)
		create.targetNHL2(np)
		create.targetIcon(np)
		create.socialIcon(np)
		create.raidIcon(np)
		create.classIcon(np)
		create.roleIcon(np)
		create.totemIcon(np)
		create.unknownIcon(np)
		create.combo(np)
		create.debug(np)
		--create.test(np)

		--create.npmouseoverHL(np)

		--create.npHLTexA(np)
		--create.npHLTexB(np)
		
		--NAME HIGHLIGHT TEXTURE (on mouseover) from -->> IF ANCHOR POINT = "CENTER"
		
		
		--NAME HIGHLIGHT TEXTURE (on mouseover) to <<--]]
	-----------------------------------------------------------------------------------
		--[[ NAME HIGHLIGHT TEXTURE (on mouseover) from -->> IF ANCHOR POINT = "CENTER"
		--local npHLTex = frame:CreateTexture(npHLTex, "HIGHLIGHT")
		npHLTex:SetTexture("Interface\\Addons\\sqwNameplates\\media\\highlight") 
		--npHLTex:SetTexture("Interface\\Addons\\sqwNameplates\\media\\highlightTargetName") 
		npHLTex:SetVertexColor(1,1,1,1)
		npHLTex:Show() 
		npHLTex.visible = true
		npHLTex:SetWidth(50)
		npHLTex:SetHeight(15)
		npHLTex:SetPoint("TOP" ,0,4)]]
		--NAME HIGHLIGHT TEXTURE (on mouseover) to <<--

		-- this is working version --
		--NAME HIGHLIGHT TEXTURE (on mouseover) from -->> IF ANCHOR POINT = "BOTTOM"
	--[[ ----------------------------------------------------------------------
		local npHLTex = frame:CreateTexture(npHLTex, "HIGHLIGHT")
		npHLTex:SetTexture("Interface\\Addons\\sqwNameplates\\media\\unknown") 
		npHLTex:SetVertexColor(1,1,1,1)
		npHLTex:Show() 
		npHLTex.visible = true
		npHLTex:SetWidth(120)
		npHLTex:SetHeight(8)
		npHLTex:SetPoint("top" ,0,-8) --if name position set to top
		--NAME HIGHLIGHT TEXTURE (on mouseover) to <<-- --]]
		
		--[[ -current from here remmed 28-06-14
	local npHLTex = frame:CreateTexture(npHLTex, "HIGHLIGHT")
		npHLTex:SetTexture("Interface\\Addons\\sqwNameplates\\media\\unknown") 
		npHLTex:SetVertexColor(1,1,1,1)
		npHLTex:Show() 
		npHLTex.visible = true
		npHLTex:SetWidth(120)
		npHLTex:SetHeight(8)
		npHLTex:SetPoint("bottom" ,0,-12) --if name position set to bottom
		
		
		--current tp here ]]
	------------------------------------------------------------
		--[[
	function create:npHLTexA()
		if not self.npHLTexA then
			npHLTex = self:CreateTexture(nil, "BORDER")
		else
			npHLTex = self.npHLTexA
		end
		npHLTex = frame:CreateTexture(npHLTex, "HIGHLIGHT")
		npHLTex:SetTexture("Interface\\Addons\\sqwNameplates\\media\\npHL") 
		npHLTex:SetVertexColor(1,1,.9,0.5)
		npHLTex:Show() 
		npHLTex.visible = true
		npHLTex:SetWidth(120)
		npHLTex:SetHeight(18)
		npHLTex:SetPoint("bottom" ,-2,2)
	   
	end
	end

		function create:npHLTexB()
		if not self.npHLTexB then
			--npHLTex = self:CreateTexture(nil, "BORDER")
		--else
			npHLTex = self.npHLTexA
		end
		npHLTex:SetTexture("Interface\\Addons\\sqwNameplates\\media\\unknown") 
		npHLTex:SetVertexColor(1,1,1,1)
		npHLTex:Show() 
		npHLTex.visible = true
		npHLTex:SetWidth(120)
		npHLTex:SetHeight(8)
		npHLTex:SetPoint("bottom" ,0,3)
	   
	end
	end]]


		------------------------------------------------------------------
		if refresh then
			local time = GetTime()
			
			wipe(np.d)
			wipe(np.savedData)
			
			local buffTime = 60
			for i=1,db.buffsNumber do
				-- Test buffs
				np.buffs[i].spellName = "Test"
				np.buffs[i].expirationTime = time+buffTime
				np.buffs[i].duration = buffTime
				np.buffs[i].count = fastrandom(0, 3)
				np.buffs[i].filter = "HELPFUL"
				
				np.buffs[i].icon:SetTexture("Interface\\Icons\\Spell_chargepositive")
				np.buffs[i].icon.tex = "Interface\\Icons\\Spell_chargepositive"
				
				np.buffs[i]:SetWidth(12)
				np.buffs[i]:SetHeight(12)
				np.buffs[i].size = 12
				
				np.buffs[i]:Show()
				np.buffs[i].visible = true
				
				-- Test debuffs
				np.debuffs[i].spellName = "Test"
				np.debuffs[i].expirationTime = time+buffTime
				np.debuffs[i].duration = buffTime
				np.debuffs[i].count = fastrandom(0, 3)
				np.debuffs[i].filter = "HARMFUL"
				
				np.debuffs[i].icon:SetTexture("Interface\\Icons\\Spell_chargenegative")
				np.debuffs[i].icon.tex = "Interface\\Icons\\Spell_chargenegative"
				
				np.debuffs[i]:SetWidth(12)
				np.debuffs[i]:SetHeight(12)
				np.debuffs[i].size = 12
				
				np.debuffs[i]:Show()
				np.debuffs[i].visible = true
			end
			
			for i=db.buffsNumber+1,12 do
				if np.buffs[i] then
					np.buffs[i]:Hide()
					np.buffs[i].visible = false
					
					np.debuffs[i]:Hide()
					np.debuffs[i].visible = false
				end
			end
		end
		
		sqwNameplates.frames[np.number] = frame
		
		frame:onShow()
	end