	local L = LibStub("AceLocale-3.0"):GetLocale("sqwNameplates")
	local LSM = LibStub("LibSharedMedia-3.0")

	local t, t2, db
	local addon = sqwNameplates


	--[[ Print ]]
	local function p(text)
		ChatFrame3:AddMessage(tostring(text))
	end
	local s = self
	addon.resources = {}

	--[[ Helper functions ]]
	-- _G global names
	addon.resources.globalNames = setmetatable({}, {__index = function(s, index)
		for k,v in pairs(_G) do
			if(v == index) then
				s[index] = k
				return k
			end
		end
		return tostring(index)
	end})

	-- Short numbers
	function addon.resources.siValue(val)
		if val >= 10000000 then 
			return format("%.1fm", val / 1000000) 
		elseif val >= 1000000 then
			return format("%.2fm", val / 1000000) 
		elseif val >= 100000 then
			return format("%.0fk", val / 1000) 
		elseif val >= 1000 then
			return format("%.1fk", val / 1000) 
		else
			return val
		end
	end

	-- Round a number
	-- number, decimal places
	function addon.resources.round(num, idp) --number, decimal places
		local mult = 10^(idp or 0)
		return math.floor(num * mult + 0.5) / mult
	end

	-- Format time
	function addon.resources.formatTime(time)
		if time>60 then
			time = floor(time/60).."m"
		else
			time = time
		end
		
		return time
	end

	-- Sort by time
	function addon.resources.sortByTime(a,b)
		local time = GetTime()
		
		if a[7]-time==b[7]-time then --[1]name, [7]expirationTime
			return a[1]>b[1]
		else
			return a[7]-time>b[7]-time
		end
	end

	-- Copy table
	function addon.resources.deepcopy(object)
		local lookup_table = {}
		local function _copy(object)
			if type(object) ~= "table" then
				return object
			elseif lookup_table[object] then
				return lookup_table[object]
			end
			local new_table = {}
			lookup_table[object] = new_table
			for index, value in pairs(object) do
				new_table[_copy(index)] = _copy(value)
			end
			return setmetatable(new_table, getmetatable(object))
		end
		return _copy(object)
	end

	-- Threat color gradient
	-- http://www.wowwiki.com/ColorGradient
	function addon.resources.colorGradient(perc, c1, c2)
		if perc>=1 then
			return c2.r, c2.g, c2.b
		elseif perc<=0 then
			return c1.r, c1.g, c1.b
		end
		
		return c1.r+(c2.r-c1.r)*perc, c1.g+(c2.g-c1.g)*perc, c1.b+(c2.b-c1.b)*perc
	end

	-- Lighten up a color
	function addon.resources.lightenColor(r, g, b)
		local modifier = 4.25 --3.85 --was 1.5 
		if not r or not g or not b then
			return r, g, b
		else
			return r+(1-r)/modifier, g+(1-g)/modifier, b+(1-b)/modifier
		end
	end
	-- Lighten guild color
	function addon.resources.lightenGuildColor(r, g, b)
		local modifier = 1.0 --was 1.5 
		if not r or not g or not b then
			return r, g, b
		else
			return r+(1-r)/modifier, g+(1-g)/modifier, b+(1-b)/modifier
		end
	end

	-- Darken a color
	function addon.resources.darkenColor(r, g, b)
		local modifier = 0.35
		
		if not r or not g or not b then
			return r, g, b
		else
			return r/modifier, g/modifier, b/modifier
		end
	end

	-- Level color
	function addon.resources.getLevelColor(level, playerLevel)
		if level>playerLevel+4 then -- Red
			return {0.75,0,0}
		elseif level>playerLevel+2 then -- Orange
			return {0.75,0.75/2,0}
		elseif playerLevel+2>=level and level>=playerLevel-2 then -- Yellow
			return {0.75,0.75,0}
		else -- Green
			if playerLevel<=9 and level>=playerLevel-4 then
				return {0,0.75,0}
			elseif playerLevel<=19 and level>=playerLevel-5 then
				return {0,0.75,0}
			elseif playerLevel<=29 and level>=playerLevel-6 then
				return {0,0.75,0}
			elseif playerLevel<=39 and level>=playerLevel-7 then
				return {0,0.75,0}
			elseif playerLevel<=70 and level>=playerLevel-8 then
				return {0,0.75,0}
			elseif playerLevel<=80 and level>=playerLevel-8 then
				return {0,0.75,0}
			elseif playerLevel<=90 and level>=playerLevel-9 then 
				return {0,0.75,0}
			else -- Grey
				return {0.5,0.5,0.5}
			end
		end
	end

	-- Social color
	function addon.resources.getSocialColor(social)
		if social=="raid" then
			return 255/255, 127/255, 0/255
		elseif social=="party" then
			return 166/255, 166/255, 255/255
		elseif social=="guild" then --GUILD COLOR
			return 64/255, 255/255, 64/255 --ORIG
			
		elseif social=="bnet" then
			return 115/255, 199/255, 255/255
		elseif social=="friend" then
			--return 154/255, 122/255, 41/255
			 return 254/255, 222/255, 80/255
			 
		else
	return 255/255, 255/255, 255/255
			
		end
	end

	--[[ Create the border ]]
	function addon:setBorder(option)
		db = addon.db.profile
		
		local size = db.borderSize
		local strata = "BACKGROUND"
		local tex
		local texSize = 16
		if option=="icon" then
			tex = db.borderIcon
		else
			tex = db.border
		end
		local r = db.borderColor.r
		local g = db.borderColor.g
		local b = db.borderColor.b
		local a = db.borderColor.a
		
		if option=="icon" then
			if not s.border then
				s.border = s:CreateTexture(nil, strata)
			end
			
			s.border:SetWidth(s:GetWidth()+size)
			s.border:SetHeight(s:GetHeight()+size)
			s.border:SetPoint("CENTER", s, "CENTER")--, size, -size)
			--s.border:SetPoint("TOPLEFT", s, "TOPLEFT", -size, size)
			--s.border:SetPoint("BOTTOMRIGHT", s, "BOTTOMRIGHT", size, -size)
			s.border:SetTexture(tex)
			s.border:SetVertexColor(r, g, b, a)
		else
			size = 0
			
			if not s.border then
				s.border = {}
				s.border.topleft = s:CreateTexture(nil, strata)
				s.border.top = s:CreateTexture(nil, strata)
				s.border.topright = s:CreateTexture(nil, strata)
				s.border.left = s:CreateTexture(nil, strata)
				s.border.right = s:CreateTexture(nil, strata)
				s.border.bottomleft = s:CreateTexture(nil, strata)
				s.border.bottom = s:CreateTexture(nil, strata)
				s.border.bottomright = s:CreateTexture(nil, strata)
			end
			
			s.border.topleft:SetWidth(texSize)
			s.border.topleft:SetHeight(texSize)
			s.border.topleft:SetPoint("BOTTOMRIGHT", s, "TOPLEFT", -size, size)
			s.border.topleft:SetTexture(tex)
			s.border.topleft:SetTexCoord(0/8, 1/8, 0, 1)
			s.border.topleft:SetVertexColor(r, g, b, a)
			
			s.border.top:SetWidth(texSize)
			s.border.top:SetHeight(texSize)
			s.border.top:SetPoint("BOTTOMLEFT", s, "TOPLEFT", -size, size)
			s.border.top:SetPoint("BOTTOMRIGHT", s, "TOPRIGHT", -size, size)
			s.border.top:SetTexture(tex)
			s.border.top:SetTexCoord(1/8, 2/8, 0, 1)
			s.border.top:SetVertexColor(r, g, b, a)
			
			s.border.topright:SetWidth(texSize)
			s.border.topright:SetHeight(texSize)
			s.border.topright:SetPoint("BOTTOMLEFT", s, "TOPRIGHT", -size, size)
			s.border.topright:SetTexture(tex)
			s.border.topright:SetTexCoord(2/8, 3/8, 0, 1)
			s.border.topright:SetVertexColor(r, g, b, a)
			
			s.border.left:SetWidth(texSize)
			s.border.left:SetHeight(texSize)
			s.border.left:SetPoint("TOPRIGHT", s, "TOPLEFT", -size, size)
			s.border.left:SetPoint("BOTTOMRIGHT", s, "BOTTOMLEFT", -size, size)
			s.border.left:SetTexture(tex)
			s.border.left:SetTexCoord(3/8, 4/8, 0, 1)
			s.border.left:SetVertexColor(r, g, b, a)
			
			s.border.right:SetWidth(texSize)
			s.border.right:SetHeight(texSize)
			s.border.right:SetPoint("TOPLEFT", s, "TOPRIGHT", -size, size)
			s.border.right:SetPoint("BOTTOMLEFT", s, "BOTTOMRIGHT", -size, size)
			s.border.right:SetTexture(tex)
			s.border.right:SetTexCoord(4/8, 5/8, 0, 1)
			s.border.right:SetVertexColor(r, g, b, a)
			
			s.border.bottomleft:SetWidth(texSize)
			s.border.bottomleft:SetHeight(texSize)
			s.border.bottomleft:SetPoint("TOPRIGHT", s, "BOTTOMLEFT", -size, size)
			s.border.bottomleft:SetTexture(tex)
			s.border.bottomleft:SetTexCoord(5/8, 6/8, 0, 1)
			s.border.bottomleft:SetVertexColor(r, g, b, a)
			
			s.border.bottom:SetWidth(texSize)
			s.border.bottom:SetHeight(texSize)
			s.border.bottom:SetPoint("TOPLEFT", s, "BOTTOMLEFT", -size, size)
			s.border.bottom:SetPoint("TOPRIGHT", s, "BOTTOMRIGHT", -size, size)
			s.border.bottom:SetTexture(tex)
			s.border.bottom:SetTexCoord(6/8, 7/8, 0, 1)
			s.border.bottom:SetVertexColor(r, g, b, a)
			
			s.border.bottomright:SetWidth(texSize)
			s.border.bottomright:SetHeight(texSize)
			s.border.bottomright:SetPoint("TOPLEFT", s, "BOTTOMRIGHT", -size, size)
			s.border.bottomright:SetTexture(tex)
			s.border.bottomright:SetTexCoord(7/8, 8/8, 0, 1)
			s.border.bottomright:SetVertexColor(r, g, b, a)
		end
	end

	--[[ Data ]]
	addon.raidIconList = {
		L["Yellow Star"],
		L["Orange Circle"],
		L["Purple Diamond"],
		L["Green Triangle"],
		L["White Moon"],
		L["Blue Square"],
		L["Red X"],
		L["White Skull"],
	}

	-- Class icon tex coords
	addon.resources.classIcon = {
		["WARRIOR"] = { 0.00, 0.25, 0.00, 0.25 },
		["MAGE"] = { 0.25, 0.50, 0.00, 0.25 },
		["ROGUE"] = { 0.50, 0.75, 0.00, 0.25 },
		["DRUID"] = { 0.75, 1.00, 0.00, 0.25 },
		["HUNTER"] = { 0.00, 0.25, 0.25, 0.50 },
		["SHAMAN"] = { 0.25, 0.50, 0.25, 0.50 },
		["PRIEST"] = { 0.50, 0.75, 0.25, 0.50 },
		["WARLOCK"] = { 0.75, 1.00, 0.25, 0.50 },
		["PALADIN"] = { 0.00, 0.25, 0.50, 0.75 },
		["DEATHKNIGHT"] = { 0.25, 0.50, 0.50, 0.75 },
		["MONK"] = { 0, 0, 1, 1 },
	}

	-- Totem spell ids
	addon.resources.totemList = {
		3599,	--Searing Totem
		2484,	--Earthbind Totem
		5394,	--Healing Stream Totem
		8190,	--Magma Totem
		8177,	--Grounding Totem
		8143,	--Tremor Totem
		2062,	--Earth Elemental Totem
		2894,	--Fire Elemental Totem
		16190,	--Mana Tide Totem
		51485,	--Earthgrab Totem
		108280,	--Healing Tide Totem
		108270,	--Stone Bulwark Totem
		108273,	--Windwalk Totem
		98008,	--Spirit Link Totem
		108269,	--Capacitor Totem
		120668,	--Stormlash Totem
		71278,	--chokingGas --"Interface\\Icons\\Ability_Creature_Cursed_01"
	}
	addon.resources.totems = {}
	t = 1
	for i=1,#addon.resources.totemList do
		local name, _, tex = GetSpellInfo(addon.resources.totemList[i])
		if name then
			addon.resources.totems[t] = { name = name, tex = tex }
			t = t+1
		end
	end
	tinsert(addon.resources.totems, { name = "Initiate's Training Dummy", tex = "Interface\\Icons\\Mail_gmicon" })

	-- Raid icon tex coords
	addon.resources.raidIconTextureCoords = {		-- ULx, ULy, LLx, LLy, URx, URy, LRx, LRy
		{ 0,0,0,0.25,0.25,0,0.25,0.25 },			--star 1
		{ 0.25,0,0.25,0.25,0.5,0,0.5,0.25 },		--circle 2
		{ 0.5,0,0.5,0.25,0.75,0,0.75,0.25 },		--purple 3
		{ 0.75,0,0.75,0.25,1,0,1,0.25 },			--triangle 4
		{ 0,0.25,0,0.5,0.25,0.25,0.25,0.5 },		--moon 5
		{ 0.25,0.25,0.25,0.5,0.5,0.25,0.5,0.5 },	--blue 6
		{ 0.5,0.25,0.5,0.5,0.75,0.25,0.75,0.5 },	--cross 7
		{ 0.75,0.25,0.75,0.5,1,0.25,1,0.5 },		--skull 8
	}

	-- Role icon tex coords
	addon.resources.roleTextureCoords = {
		TANK = { 0.5, 0.75, 0, 1.0 },
		HEALER = { 0.75, 1.0, 0, 1.0 },
		DAMAGER = { 0.25, 0.5, 0, 1.0 },
	}

	-- Class colors
	addon.resources.classByColor = {
		C285 = "DEATHKNIGHT",
		C390 = "DRUID",
		C498 = "HUNTER",
		C549 = "MAGE",
		C405 = "MONK",
		C571 = "PALADIN",
		C765 = "PRIEST",	
		C605 = "ROGUE",	
		C334 = "SHAMAN",	
		C479 = "WARLOCK",
		C465 = "WARRIOR",
		--C256 = "Npc",
	}

	-- ToC: Faction Champs
	function addon.resources.factionChampClassAndRole(name)
		--[[Shaman]]--
		if name==L["Saamul"] or name==L["Thrakgar"] then
			return "SHAMAN", "DAMAGER"
		elseif name==L["Broln Stouthorn"] or name==L["Shaabad"] then
			return "SHAMAN", "HEALER"
			--[[Paladins]]--
		elseif name==L["Baelnor Lightbearer"] or name==L["Malithas Brightblade"] then
			return "PALADIN", "TANK"
		elseif name==L["Velanaa"] or name==L["Liandra Suncaller"] then
			return "PALADIN", "HEALER"
			--[[Druids]]--
		elseif name==L["Kavina Grovesong"] or name==L["Birana Stormhoof"] then
			return "DRUID", "DAMAGER"
		elseif name==L["Melador Valestrider"] or name==L["Erin Misthoof"] then
			return "DRUID", "HEALER"
			--[[Priests]]--
		elseif name==L["Brienna Nightfell"] or name==L["Vivienne Blackwhisper"] then
			return "PRIEST", "DAMAGER"
		elseif name==L["Anthar Forgemender"] or name==L["Caiphus the Stern"] then
			return "PRIEST", "HEALER"
			--[[Warlocks]]--
		elseif name==L["Serissa Grimdabbler"] or name==L["Harkzog"] then
			return "WARLOCK", "DAMAGER"
			--[[Warriors]]--
		elseif name==L["Shocuul"] or name==L["Narrhok Steelbreaker"] then
			return "WARRIOR", "TANK"
			--[[Rogues]]--
		elseif name==L["Irieth Shadowstep"] or name==L["Maz'dinah"] then
			return "ROGUE", "DAMAGER"
			--[[Death Knights]]--
		elseif name==L["Tyrius Duskblade"] or name==L["Gorgrim Shadowcleave"] then
			return "DEATHKNIGHT", "TANK"
			--[[Hunters]]--
		elseif name==L["Alyssia Moonstalker"] or name==L["Ruj'kah"] then
			return "HUNTER", "DAMAGER"
			--[[Mages]]--
		elseif name==L["Noozle Whizzlestick"] or name==L["Ginselle Blightslinger"] then
			return "MAGE", "DAMAGER"
		else
			return nil
		end
	end